@@ -250,17 +250,29 @@ class BaseRouter(ABC):
                 ]
             ]
 
+        src.append(Line("raise NotFound", 1))
+
         self.optimize(src)
 
         self.find_route_src = "".join(
             map(str, filter(lambda x: x.render, src))
         )
         if do_compile:
-            compiled_src = compile(
-                self.find_route_src,
-                "",
-                "exec",
-            )
+            try:
+                compiled_src = compile(
+                    self.find_route_src,
+                    "",
+                    "exec",
+                )
+            except SyntaxError as se:
+                syntax_error = (
+                    f"Line {se.lineno}: {se.msg}\n{se.text}"
+                    f"{' '*max(0,int(se.offset or 0)-1) + '^'}"
+                )
+                raise FinalizationError(
+                    f"Cannot compile route AST:\n{self.find_route_src}"
+                    f"\n{syntax_error}"
+                )
             ctx: t.Dict[t.Any, t.Any] = {}
             exec(compiled_src, None, ctx)
             self._find_route = ctx["find_route"]
@@ -289,13 +301,17 @@ class BaseRouter(ABC):
         for num, line in enumerate(src):
             if line.indent < current:
                 if not line.src.startswith("."):
-                    offset = 0
+                    if offset < 0:
+                        offset += 1
+                    else:
+                        offset = 0
 
             if (
                 line.src.startswith("if")
                 or line.src.startswith("elif")
                 or line.src.startswith("return")
                 or line.src.startswith("basket")
+                or line.src.startswith("try")
             ):
 
                 idnt = line.indent + 1
