@@ -2,7 +2,7 @@ import typing as t
 from logging import getLogger
 
 from .line import Line
-from .patterns import REGEX_PARAM_NAME
+from .patterns import REGEX_PARAM_NAME, REGEX_TYPES
 from .route import Route
 
 logger = getLogger("sanic.root")
@@ -18,6 +18,7 @@ class Node:
         self._children: t.Dict[str, "Node"] = {}
         self.children: t.Dict[str, "Node"] = {}
         self.level = 0
+        self.offset = 0
         self.route: t.Optional[Route] = None
         self.dynamic = False
         self.first = False
@@ -63,8 +64,15 @@ class Node:
         output += delayed
         return output
 
+    def apply_offset(self, amt, apply_self=True, apply_children=False):
+        if apply_self:
+            self.offset += amt
+        if apply_children:
+            for child in self.children.values():
+                child.apply_offset(amt, apply_children=True)
+
     def to_src(self) -> t.Tuple[t.List[Line], t.List[Line]]:
-        indent = (self.level + 1) * 2 - 3
+        indent = (self.level + 1) * 2 - 3 + self.offset
         delayed: t.List[Line] = []
         src: t.List[Line] = []
 
@@ -73,19 +81,29 @@ class Node:
         return_bump = 1
 
         if self.first or self.root:
+            src = []
             operation = ">"
             use_level = level
+            conditional = "if"
             if (
                 self.last
                 and self.route
-                # and not self.level == 1
                 and not self.children
                 and not self.route.requirements
             ):
                 use_level = self.level
                 operation = "=="
                 equality_check = True
-            src = [Line(f"if num {operation} {use_level}:", indent)]
+                conditional = "elif"
+                src.extend(
+                    [
+                        Line(f"if num > {use_level}:", indent),
+                        Line("raise NotFound", indent + 1),
+                    ]
+                )
+            src.append(
+                Line(f"{conditional} num {operation} {use_level}:", indent)
+            )
 
         if self.dynamic:
             if not self.parent.children_basketed:
@@ -93,9 +111,10 @@ class Node:
                 src.append(
                     Line(f"basket[{level}] = parts[{level}]", indent + 1)
                 )
-                # This is a control line to help control indentation, but
-                # it should not be rendered
-                src.append(Line("...", 0, offset=-1, render=False))
+                self.parent.apply_offset(-1, False, True)
+
+                if not self.children:
+                    return_bump -= 1
         else:
             if_stmt = "if" if self.first or self.root else "elif"
             len_check = (
@@ -139,17 +158,15 @@ class Node:
                         ),
                         return_indent,
                     ),
+                    # Line("...", return_indent - 1, render=True),
                 ]
             )
-            # This is a control line to help control indentation, but
-            # it should not be rendered
-            location.append(
-                Line(
-                    "...",
-                    indent + return_bump + bool(not self.children),
-                    render=False,
-                )
-            )
+            if self.route.params:
+                location.append(Line("...", return_indent - 1, render=False))
+                if self.last:
+                    location.append(
+                        Line("...", return_indent - 2, render=False),
+                    )
         return src, delayed
 
     def add_child(self, child: "Node") -> None:
@@ -173,8 +190,10 @@ class Node:
 
     def _inject_params(self, location, indent):
         lines = [
-            Line("try:", indent),
+            Line(f"if num > {self.level}:", indent),
+            Line("raise NotFound", indent + 1),
         ]
+        lines.append(Line("try:", indent))
         for idx, param in self.route.params.items():
             unquote_start = "unquote(" if self.route.unquote else ""
             unquote_end = ")" if self.route.unquote else ""
@@ -191,15 +210,24 @@ class Node:
             lines
             + [
                 Line("except ValueError:", indent),
-                Line("...", indent + 1),
+                Line("pass", indent + 1),
                 Line("else:", indent),
             ]
         )
 
     @staticmethod
-    def _sorting(item) -> t.Tuple[bool, int, str]:
+    def _sorting(item) -> t.Tuple[bool, int, str, int]:
         key, child = item
-        return child.dynamic, len(child._children) * -1, key
+        type_ = 0
+        if child.dynamic:
+            key = key[1:-1]
+            if ":" in key:
+                key, param_type = key.split(":")
+                try:
+                    type_ = list(REGEX_TYPES.keys()).index(param_type)
+                except ValueError:
+                    type_ = len(list(REGEX_TYPES.keys()))
+        return child.dynamic, len(child._children) * -1, key, type_ * -1
 
 
 class Tree:
